package com.syc.slm.common.core.util;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.google.common.collect.Maps;
import com.syc.slm.common.core.constant.SlmConstants;
import com.syc.slm.slmbi.exception.SysException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.*;
import org.springframework.web.client.RestTemplate;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Type;
import java.util.Map;
import java.util.function.Supplier;

/**
 * slm-bi
 * RestTemplate 远程调用工具类
 *
 * @author : lin
 * @date : 2021-04-13 10:15
 **/
@Slf4j
public final class RestTemplateUtils {

    private static final RestTemplate REST_TEMPLATE = new RestTemplate();

    public static <TResult> TResult get(String url, final Type resultType, Map<String, ?> uriVariables) {
        return get(url, null, resultType, uriVariables);
    }

    public static <TResult> TResult get(String url, Supplier<Map<String, String>> headerHandler,  final Type resultType, Map<String, ?> uriVariables) {
        try {
            // 获取token以及在请求头加上token
            HttpServletRequest request = ServletUtils.getHttpServletRequest();
            String token = request.getHeader(SlmConstants.TOKEN_PREFIX);
            if (StringUtils.isBlank(token)) {
                throw new SysException("token不能为空");
            }
            Map<String, String> headMap = Maps.newHashMap();
            headMap.put(SlmConstants.TOKEN_PREFIX, token);
            if (headerHandler != null) {
                Map<String, String> header = headerHandler.get();
                header.forEach(headMap::put);
            }
            HttpHeaders httpHeaders = new HttpHeaders();
            httpHeaders.setAll(headMap);
            log.info("GET请求，开始请求DC，uri是{},参数是{}",url,SlmConstants.gson.toJson(uriVariables));
            HttpEntity<?> requestEntity = new HttpEntity<>(httpHeaders);
            ResponseEntity<String> response = REST_TEMPLATE.exchange(url,  HttpMethod.GET, requestEntity, String.class, uriVariables);
            log.info("GET请求结果是=============:{}", response.getBody());
            if (HttpStatus.OK.value() == response.getStatusCodeValue()) {
                if (resultType.equals(String.class)) {
                    return (TResult)response.getBody();
                }
                return SlmConstants.gson.fromJson(response.getBody(), resultType);
            }
        } catch (Exception e) {
            log.error("出错啦，{}", e);
            throw new SysException("出错啦，请联系管理员", e.toString());
        }
        return null;
    }


    public static <TResult> TResult post(String url, Object requestBody, final Type resultType, Map<String, ?> uriVariables) {
        return post(url, null, requestBody, resultType, uriVariables);
    }


    public static <TResult> TResult post(String url, Supplier<Map<String, String>> headerHandler,  Object requestBody, final Type resultType, Map<String, ?> uriVariables) {
        try {
            // 获取token以及在请求头加上token
            HttpServletRequest request = ServletUtils.getHttpServletRequest();
            String token = request.getHeader(SlmConstants.TOKEN_PREFIX);
            if (StringUtils.isBlank(token)) {
                throw new SysException("token不能为空");
            }
            Map<String, String> headMap = Maps.newHashMap();
            headMap.put(SlmConstants.TOKEN_PREFIX, token);
            if (headerHandler != null) {
                Map<String, String> header = headerHandler.get();
                header.forEach(headMap::put);
            }

            HttpHeaders httpHeaders = new HttpHeaders();
            httpHeaders.setAll(headMap);

            log.info("POST请求，开始请求DC，uri是{},参数是{}",url,SlmConstants.gson.toJson(requestBody));
            HttpEntity<Object> requestEntity = new HttpEntity<>(requestBody, httpHeaders);
            ResponseEntity<String> response =REST_TEMPLATE.exchange(url, HttpMethod.POST, requestEntity, String.class, uriVariables);
            log.info("POST请求结束，结果是:{}",response.toString());
            if (HttpStatus.OK.value() == response.getStatusCodeValue()) {
                if (resultType.equals(String.class)) {
                    return (TResult)response.getBody();
                }
                return SlmConstants.gson.fromJson(response.getBody(), resultType);
            }
        } catch (Exception e) {
            log.error("出错啦，{}", e);
            throw new SysException("出错啦，请联系管理员", e.toString());
        }
        return null;
    }


    /**
     * 带请求头的GET请求调用方式
     *
     * @param url          请求URL
     * @param headers      请求头参数
     * @param responseType 返回对象类型
     * @param uriVariables URL中的变量，与Map中的key对应
     * @return ResponseEntity 响应对象封装类
     */
    public static <T> ResponseEntity<T> get(String url,
        Map<String, String> headers,
        Class<T> responseType,
        Map<String, ?> uriVariables) {
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setAll(headers);
        return get(url, httpHeaders, responseType, uriVariables);
    }

    /**
     * 带请求头的GET请求调用方式
     *
     * @param url          请求URL
     * @param headers      请求头参数
     * @param responseType 返回对象类型
     * @param uriVariables URL中的变量，与Map中的key对应
     * @return ResponseEntity 响应对象封装类
     */
    public static <T> ResponseEntity<T> get(String url,
        HttpHeaders headers,
        Class<T> responseType,
        Map<String, ?> uriVariables) {
        HttpEntity<?> requestEntity = new HttpEntity<>(headers);
        return exchange(url, HttpMethod.GET, requestEntity, responseType, uriVariables);
    }

    /**
     * POST请求调用方式
     *
     * @param url          请求URL
     * @param responseType 返回对象类型
     * @return
     */
    public static <T> ResponseEntity<T> post(String url, Class<T> responseType) {
        return REST_TEMPLATE.postForEntity(url, HttpEntity.EMPTY, responseType);
    }

    /**
     * 带请求头的POST请求调用方式
     *
     * @param url          请求URL
     * @param headers      请求头参数
     * @param requestBody  请求参数体
     * @param responseType 返回对象类型
     * @param uriVariables URL中的变量，与Map中的key对应
     * @return ResponseEntity 响应对象封装类
     */
    public static <T> ResponseEntity<T> post(String url,
        Map<String, String> headers,
        Object requestBody,
        Class<T> responseType,
        Map<String, ?> uriVariables) {
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setAll(headers);
        return post(url, httpHeaders, requestBody, responseType, uriVariables);
    }

    /**
     * 带请求头的POST请求调用方式
     *
     * @param url          请求URL
     * @param headers      请求头参数
     * @param requestBody  请求参数体
     * @param responseType 返回对象类型
     * @param uriVariables URL中的变量，与Map中的key对应
     * @return ResponseEntity 响应对象封装类
     */
    public static <T> ResponseEntity<T> post(String url,
        HttpHeaders headers,
        Object requestBody,
        Class<T> responseType,
        Map<String, ?> uriVariables) {
        HttpEntity<Object> requestEntity = new HttpEntity<Object>(requestBody, headers);
        return post(url, requestEntity, responseType, uriVariables);
    }

    /**
     * 自定义请求头和请求体的POST请求调用方式
     *
     * @param url           请求URL
     * @param requestEntity 请求头和请求体封装对象
     * @param responseType  返回对象类型
     * @param uriVariables  URL中的变量，与Map中的key对应
     * @return ResponseEntity 响应对象封装类
     */
    public static <T> ResponseEntity<T> post(String url,
        HttpEntity<?> requestEntity,
        Class<T> responseType,
        Map<String, ?> uriVariables) {
        return REST_TEMPLATE.exchange(url, HttpMethod.POST, requestEntity, responseType, uriVariables);
    }

    /**
     * 通用调用方式
     *
     * @param url           请求URL
     * @param method        请求方法类型
     * @param requestEntity 请求头和请求体封装对象
     * @param responseType  返回对象类型
     * @param uriVariables  URL中的变量，与Map中的key对应
     * @return ResponseEntity 响应对象封装类
     */
    public static <T> ResponseEntity<T> exchange(String url,
        HttpMethod method,
        HttpEntity<?> requestEntity,
        Class<T> responseType,
        Map<String, ?> uriVariables) {
        return REST_TEMPLATE.exchange(url, method, requestEntity, responseType, uriVariables);
    }

    //#endregion

    /**
     * 获取RestTemplate实例对象，可自由调用其方法
     *
     * @return RestTemplate实例对象
     */
    public static RestTemplate getRestTemplate() {
        return REST_TEMPLATE;
    }

}